/*
Golf 2D Game

Files:
    engine\golf2d\g2d_eng.cpp
    engine\golf2d\g2d_eng.h

Classes:
    G2DEngine   The engine for the Golf 2D game.

Credits and Copyright:
    Copyright 2002, Andrew Russell. All Rights Reserved.
*/

#ifndef ARSSC_ENGINE_GOLF2D_G2D_ENG_H_INCLUDED
#define ARSSC_ENGINE_GOLF2D_G2D_ENG_H_INCLUDED

#include "engine\engine.h"

#if (ARSSC_USE_ENGINE_CLASS == G2DEngine)
#define ARSSC_USING_ENGINE_GOLF2D

#include "engine\golf2d\g2d_lev.h"
#include "glfont\glfont.h"
#include "input\input.h"
#include "render\vector.h"
#include "utils/framecnt.h"

/// The diffrent game states.
enum G2DEngineState
{
    /// Wrong State!? (how to break the game)
    G2D_ES_NOSTATE,

    /// Playing the introduction.
    G2D_ES_INTRO,

    /// Displaying the main menu, outside of the game.
    G2D_ES_MAINMENU,

    /// Displaying the credits
    G2D_ES_CREDITS

};

/// The Golf 2D Engine. This takes care of everything related to the golf game.
class G2DEngine : public AEngineBase
{
public:
    /// Construct
    G2DEngine();

    /// Deconstruct
    virtual ~G2DEngine();

    /// Load up the engine
    virtual void Load();

    /// Close down the engine
    virtual void Unload();

    /// Load OpenGL related data (for full screen change)
    virtual void LoadOGL();
    /// Unload OpenGL related data (for full screen change)
    virtual void UnloadOGL();

    /// Update the engine
    virtual void Update(DWORD count);

    /// Draw to the screen
    virtual void Draw();

private:

    /// A map of the keyboard (AInputKeyboardKey[256])
    AInputKeyboardKey* Keyboard;

    /// The current state of the engine
    G2DEngineState m_State;

    G2DLevel* level;

    /// The main font.
    AOGLFont* font;
    /// The texture of the font
    AOGLTexture* fonttexture;

    AFrameCounter framecount;

};

#endif // (ARSSC_USE_ENGINE_CLASS == G2DEngine)

#endif // ARSSC_ENGINE_GOLF2D_G2D_ENG_H_INCLUDED
