#include "engine\golf2d\g2d_lev.h"
#include "math.h"

#pragma message IMPORTANT:: FINISH MOVING STUFF TO LoadOGL and UnLoadOGL!!!

void G2DLevel::Load(AString file)
{
    clearcolour = AV4f(0.25f, 0.75f, 1.0f, 1.0f);

    suntexture.InitLoadTGA(A_STR("data\\textures\\sun.tga"));
    mtntexture.InitLoadTGA(A_STR("data\\textures\\mtns.tga"));
    cloudtexture.InitLoadTGA(A_STR("data\\textures\\cloud.tga"));
    landtexture.InitLoadTGA(A_STR("data\\textures\\land.tga"));
    grasstexture.InitLoadTGA(A_STR("data\\textures\\grass.tga"));
    watertexture.InitLoadTGA(A_STR("data\\textures\\water.tga"));
    causticstexture.InitLoadTGA(A_STR("data\\textures\\caustic.tga"));
    rays1texture.InitLoadTGA(A_STR("data\\textures\\rays1.tga"));
    rays2texture.InitLoadTGA(A_STR("data\\textures\\rays2.tga"));  

    level_min_x = 0.0f;
    level_min_y = 0.0f;
    level_max_x = 4096.0f;
    level_max_y = 1200.0f;

    // Setup the sun
    suncenter = AV2f(200.0f,700.0f);
    sunradius = 128.0f;

    // Setup the mountians
    mtncount = 2;
    mountians = new G2DMountian[mtncount];
    
    mountians[0].texturenumber = 0;
    mountians[0].Pos = AV2f(500.0f, 300.f);

    mountians[1].texturenumber = 2;
    mountians[1].Pos = AV2f(1500.0f, 420.0f);

    // Setup the clouds
    windspeed = 0.1f;

    cloudcount = 4;
    clouds = new G2DCloud[cloudcount];
    
    clouds[0].texturenumber = 0;
    clouds[0].Pos = AV2f(800.0f, 700.f);

    clouds[1].texturenumber = 1;
    clouds[1].Pos = AV2f(1800.0f, 620.0f);

    clouds[2].texturenumber = 0;
    clouds[2].Pos = AV2f(2400.0f, 500.0f);

    clouds[3].texturenumber = 1;
    clouds[3].Pos = AV2f(0.0f, 550.0f);

    // Setup the landscape
    landsize = 64;
    landheights1 = new char[landsize];
    landheights2 = new char[landsize];

    for(unsigned int i = 0; i<landsize; i++)
    {
        landheights1[i] = 4;
        landheights2[i] = 4;
    }

    landheights2[8]=3;

    landheights1[9]=3;
    landheights2[9]=3;

    landheights1[10]=3;
    landheights2[10]=5;

    landheights1[11]=5;
    landheights2[11]=5;

    landheights1[12]=5;
    landheights2[12]=3;

    landheights1[13]=3;

    landheights1[2]=0;
    landheights1[3]=0;
    landheights1[4]=0;
    landheights1[5]=0;

    // Setup the Water
    watercount = 3;
    water = new G2DWater[watercount];
    water[0].PosX = 2;
    water[0].PosY = 0;
    water[0].SizeX = 4;
    water[0].SizeY = 4;

    water[1].PosX = 16;
    water[1].PosY = 0;
    water[1].SizeX = 6;
    water[1].SizeY = 4;

    water[2].PosX = 30;
    water[2].PosY = 0;
    water[2].SizeX = 2;
    water[2].SizeY = 4;

    glClearColor(clearcolour.vR, clearcolour.vG, clearcolour.vB, clearcolour.vA);
}

void G2DLevel::Unload()
{
    delete[] mountians;
    delete[] clouds;
}

void G2DLevel::Update(DWORD count)
{
    // check position
    if(cameratrans.vX > -level_min_x) {cameratrans.vX = -level_min_x;}
    else if(cameratrans.vX < -(level_max_x-1024)) {cameratrans.vX = -(level_max_x-1024);}

    if(cameratrans.vY > -level_min_y) {cameratrans.vY = -level_min_y;}
    else if(cameratrans.vY < -(level_max_y-768)) {cameratrans.vY = -(level_max_y-768);}

    // blow clouds
    for(int i = 0; i < cloudcount; i++)
    {
        clouds[i].Pos.vX+=windspeed*0.25*count;  // clouds go at 1/4 of actual wind speed to simulate distance

        // *0.6f :: the translation modifier for the clouds. (ie: the size of the cloud layer is smaller than the actual level size)
        if(clouds[i].Pos.vX > (level_max_x*0.6f)+256) {clouds[i].Pos.vX = (level_min_x*0.6f)-256;}
        else if(clouds[i].Pos.vX < (level_min_x*0.6f)-256) {clouds[i].Pos.vX = (level_max_x*0.6f)+256;}
    }


    waterphase+=0.001f*count;
}

void G2DLevel::Draw()
{
    DrawSun();
    DrawMountians();
    DrawClouds();
    DrawLand();
    DrawWater();
}

void G2DLevel::DrawSun()
{
    glLoadIdentity();

    (cameratrans*0.2f).glTrn();

    suntexture.UseTexture();

    glBegin(GL_QUADS);
        glTexCoord2f(0,1);
        glVertex2i( suncenter.vX-sunradius, suncenter.vY-sunradius);
        glTexCoord2f(1,1);
        glVertex2i( suncenter.vX+sunradius, suncenter.vY-sunradius);
        glTexCoord2f(1,0);
        glVertex2i( suncenter.vX+sunradius, suncenter.vY+sunradius);
        glTexCoord2f(0,0);
        glVertex2i( suncenter.vX-sunradius, suncenter.vY+sunradius);
    glEnd();

}

void G2DLevel::DrawMountians()
{
    glLoadIdentity();
    (cameratrans*0.5f).glTrn();

    mtntexture.UseTexture();

    for(int i = 0; i < mtncount; i++)
    {
        mountians[i].Pos.glTrn();

        glBegin(GL_QUADS);
            glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
            glTexCoord2f(0.0f, (mountians[i].texturenumber+1)*0.25f );
            glVertex2i( 0.0f, 0.0f);
            glTexCoord2f(1.0f, (mountians[i].texturenumber+1)*0.25f );
            glVertex2i( 512.0f, 0.0f);
            glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
            glTexCoord2f(1.0f, (mountians[i].texturenumber)*0.25f+0.01f);
            glVertex2i( 512.0f, 128.0f);
            glTexCoord2f(0.0f, (mountians[i].texturenumber)*0.25f+0.01f);
            glVertex2i( 0.0f, 128.0f);

        glEnd();

        (-(mountians[i].Pos)).glTrn();
    }
}

void G2DLevel::DrawClouds()
{
    glLoadIdentity();
    (cameratrans*0.6f).glTrn();

    cloudtexture.UseTexture();

    for(int i = 0; i < cloudcount; i++)
    {
        clouds[i].Pos.glTrn();
       
        glBegin(GL_QUADS);

            glTexCoord2f(0.0f, (clouds[i].texturenumber+1)*0.5f );
            glVertex2i( 0.0f, 0.0f);
            glTexCoord2f(1.0f, (clouds[i].texturenumber+1)*0.5f );
            glVertex2i( 256.0f, 0.0f);
            glTexCoord2f(1.0f, (clouds[i].texturenumber)*0.5f);
            glVertex2i( 256.0f, 128.0f);
            glTexCoord2f(0.0f, (clouds[i].texturenumber)*0.5f);
            glVertex2i( 0.0f, 128.0f);

        glEnd();

        (-(clouds[i].Pos)).glTrn();
    }

}

void G2DLevel::DrawLand()
{
    glLoadIdentity();
    cameratrans.glTrn();

    DrawDarkGrass();

    landtexture.UseTexture();

    glBegin(GL_QUADS);

    for(unsigned int i = 0; i<landsize; i++)
    {
        if(landheights1[i] > 0)
        {
            glTexCoord2f(0.25f*i,       0.0f );
            glVertex2i( 64.0f*i,        0.0f);
            glTexCoord2f(0.25f*(i+1),   0.0f );
            glVertex2i( 64.0f*(i+1),    0.0f);
            glTexCoord2f(0.25f*(i+1),   (landheights2[i])*0.25f);
            glVertex2i( 64.0f*(i+1),    (landheights2[i])*64.0f);
            glTexCoord2f(0.25f*i,       (landheights1[i])*0.25f);
            glVertex2i( 64.0f*i,        (landheights1[i])*64.0f);
        }
    }

    glEnd();

    DrawGrass();
}

void G2DLevel::DrawGrass()
{
    grasstexture.UseTexture();

    glBegin(GL_QUADS);

    for(unsigned int i = 0; i<landsize; i++)
    {
        if(landheights1[i] > 0)
        {

            glTexCoord2f(i*0.5f,        0.0f);
            glVertex2i( 64.0f*i,        (landheights1[i])*64.0f-64.0f);
            glTexCoord2f((i+1)*0.5f,    0.0f );
            glVertex2i( 64.0f*(i+1),    (landheights2[i])*64.0f-64.0f);
            glTexCoord2f((i+1)*0.5f,    1.0f );
            glVertex2i( 64.0f*(i+1),    (landheights2[i])*64.0f+64.0f);
            glTexCoord2f(i*0.5f,        1.0f );
            glVertex2i( 64.0f*i,        (landheights1[i])*64.0f+64.0f);
        }
    }

    glEnd();
}

void G2DLevel::DrawDarkGrass()
{
    glTranslatef(0.0f, (cameratrans.vY*0.01f+3.0f),0.0f);
    glColor3f(0.5f,0.5f,0.5f);

    grasstexture.UseTexture();

    glBegin(GL_QUADS);

    for(unsigned int i = 0; i<landsize; i++)
    {
        if(landheights1[i] > 0)
        {

            glTexCoord2f((i+1)*0.5f,    0.0f);
            glVertex2i( 64.0f*i,        (landheights1[i])*64.0f-64.0f);
            glTexCoord2f(i*0.5f,        0.0f );
            glVertex2i( 64.0f*(i+1),    (landheights2[i])*64.0f-64.0f);
            glTexCoord2f(i*0.5f,        1.0f );
            glVertex2i( 64.0f*(i+1),    (landheights2[i])*64.0f+64.0f);
            glTexCoord2f((i+1)*0.5f,    1.0f );
            glVertex2i( 64.0f*i,        (landheights1[i])*64.0f+64.0f);
        }
    }

    glEnd();

    glColor3f(1.0f,1.0f,1.0f);
    glTranslatef(0.0f,-(cameratrans.vY*0.01f+3.0f),0.0f);

}

void G2DLevel::DrawWater()
{
    for(int i = 0; i<watercount; i++)
    {

        glLoadIdentity();
        cameratrans.glTrn();

        glTranslatef( (water[i].PosX)*(64), (water[i].PosY)*(64), 0.5f);
        
        // GRASS
        glTranslatef(0.0f, ((cameratrans.vY*0.01f)+3.0f+(water[i].SizeY*64.0f)),0.0f);

        glColor3f(0.5,0.5f,0.5f);
        grasstexture.UseTexture();

        glBegin(GL_QUADS);

        for(int j = 0; j<water[i].SizeX; j++)
        {
            glTexCoord2f((j+1)*0.5f,     0.0f );
            glVertex2i( 64.0f*j,        -64.0f);
            glTexCoord2f(j*0.5f,         0.0f );
            glVertex2i( 64.0f*(j+1),    -64.0f);
            glTexCoord2f(j*0.5f,         1.0f );
            glVertex2i( 64.0f*(j+1),     64.0f);
            glTexCoord2f((j+1)*0.5f,     1.0f );
            glVertex2i( 64.0f*j,         64.0f);
        }

        glEnd();

        glTranslatef(0.0f, -((cameratrans.vY*0.01f)+3.0f+(water[i].SizeY*64.0f)),0.0f);

        // BACKGROUND

        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);

        glColor3f(1.0,1.0f,1.0f);

        watertexture.UseTexture();

        glBegin(GL_QUADS);

            glTexCoord2f(0.0f, 0.0f);
            glVertex2i( 0.0f, 0.0f);
            glTexCoord2f(water[i].SizeX*0.25f, 0.0f);
            glVertex2i( water[i].SizeX*64.0f, 0.0f);
            glTexCoord2f( water[i].SizeX*0.25f, water[i].SizeY*0.25f);
            glVertex2i( water[i].SizeX*64.0f, water[i].SizeY*64.0f);
            glTexCoord2f( 0.0f, water[i].SizeY*0.25f);
            glVertex2i( 0.0f, water[i].SizeY*64.0f);

        glEnd();

        // CAUSTICS

        glBlendFunc(GL_SRC_ALPHA,GL_ONE); // Additive Blending

        glColor4f(1.0,1.0f,1.0f,0.35f);

        causticstexture.UseTexture();

        float texmod = cos(waterphase)*0.125f;

        glBegin(GL_QUADS);

            glTexCoord2f( texmod+waterphase*0.05f, waterphase*0.02f-texmod);
            glVertex2i( 0.0f, 0.0f);
            glTexCoord2f( water[i].SizeY*0.5f-texmod+waterphase*0.05f, texmod+waterphase*0.02f);
            glVertex2i( water[i].SizeX*64.0f, 0.0f);
            glTexCoord2f( water[i].SizeX*0.5f+texmod+waterphase*0.05f, water[i].SizeY*0.5f-texmod+waterphase*0.02f);
            glVertex2i( water[i].SizeX*64.0f, water[i].SizeY*64.0f);
            glTexCoord2f( waterphase*0.05f-texmod, water[i].SizeY*0.5f+texmod+waterphase*0.02f);
            glVertex2i( 0.0f, water[i].SizeY*64.0f);

        glEnd();


        // RAYS

        glDepthFunc(GL_GEQUAL);

        glTranslatef( ((water[i].SizeX)*64)/2, ((water[i].SizeY)*64)+32, -0.25f);

        glRotatef(waterphase*10.25f, 0.0f, 0.0f, 1.0f);

        float rayscale = (water[i].SizeX)/4.0f;

        glScalef(rayscale,rayscale,rayscale);

        glColor4f(1.0,1.0f,1.0f,0.3f+texmod);
        rays1texture.UseTexture();

        glBegin(GL_QUADS);
            glTexCoord2f(0,1);
            glVertex2i( -256.0f, -256.0f);
            glTexCoord2f(1,1);
            glVertex2i(  256.0f, -256.0f);
            glTexCoord2f(1,0);
            glVertex2i(  256.0f,  256.0f);
            glTexCoord2f(0,0);
            glVertex2i( -256.0f,  256.0f);
        glEnd();

        glRotatef(waterphase*(-17.0f), 0.0f, 0.0f, 1.0f);  // and back the other way!

        glColor4f(1.0,1.0f,1.0f,0.8f);
        rays2texture.UseTexture();

        glBegin(GL_QUADS);
            glTexCoord2f(0,1);
            glVertex2i( -256.0f, -256.0f);
            glTexCoord2f(1,1);
            glVertex2i(  256.0f, -256.0f);
            glTexCoord2f(1,0);
            glVertex2i(  256.0f,  256.0f);
            glTexCoord2f(0,0);
            glVertex2i( -256.0f,  256.0f);
        glEnd();

        glDisable(GL_DEPTH_TEST);

        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); // Reset Blending
        
        glColor4f(1.0,1.0f,1.0f,1.0f);

    }
}

