/*
Golf 2D Game

Files:
    engine\golf2d\g2d_lev.cpp
    engine\golf2d\g2d_lev.h

Classes:
    G2DLevel    A level in the G2D game.

Credits and Copyright:
    Copyright 2002, Andrew Russell. All Rights Reserved.
*/

#ifndef ARSSC_ENGINE_GOLF2D_G2D_LEV_H_INCLUDED
#define ARSSC_ENGINE_GOLF2D_G2D_LEV_H_INCLUDED

#include <windows.h>

#include "utils\stexman.h"
#include "render\vector.h"

class G2DMountian
{
public:
    int texturenumber;
    AV2f Pos;   // x       y
};

class G2DCloud
{
public:
    int texturenumber;
    AV2f Pos;   // x       y
};

class G2DWater
{
public:
    int PosX, PosY, SizeX, SizeY;
};

/// A hole of the course.
class G2DLevel
{
public:
    /// Load up the level
    void Load(AString file);

    /// Close down the level
    void Unload();

    /// Load OpenGL related data (for full screen change)
    void LoadOGL();
    /// Unload OpenGL related data (for full screen change)
    void UnloadOGL();

    /// Update the level
    void Update(DWORD count);

    /// Draw the level
    void Draw();

    inline void MoveUp(float amt) {cameratrans.vY += amt;}
    inline void MoveDn(float amt) {cameratrans.vY -= amt;}
    inline void MoveLf(float amt) {cameratrans.vX -= amt;}
    inline void MoveRt(float amt) {cameratrans.vX += amt;}

private:

    inline void DrawSun();
    inline void DrawMountians();
    inline void DrawClouds();
    inline void DrawLand();
    inline void DrawGrass();
    inline void DrawDarkGrass();
    inline void DrawWater();

    AV2f cameratrans;
    float level_min_x;
    float level_max_x;
    float level_min_y;
    float level_max_y;

    //TEXTURES
    AOGLTexture suntexture;
    AOGLTexture mtntexture;
    AOGLTexture cloudtexture;
    AOGLTexture landtexture;
    AOGLTexture grasstexture;
    AOGLTexture watertexture;
    AOGLTexture causticstexture;
    AOGLTexture rays1texture;
    AOGLTexture rays2texture;

    //SKY
    /// Clear Colour
    AV4f clearcolour;

    //SUN
    /// The center of the sun
    AV2f suncenter;
    /// Half the size of the sun
    float sunradius;

    //MOUNTIANS
    int mtncount;
    G2DMountian* mountians;

    //CLOUDS
    float windspeed;
    int cloudcount;
    G2DCloud* clouds;

    //LANDSCAPE
    unsigned int landsize;
    char* landheights1;
    char* landheights2;

    //Water
    int watercount;
    G2DWater* water;
    float waterphase;
};

#endif // ARSSC_ENGINE_GOLF2D_G2D_LEV_H_INCLUDED
